use libmdbx::{Database, Table, TransactionKind, WriteFlags, WriteMap, RW};
use ouroboros::self_referencing;
use std::borrow::Cow;

use crate::environment::Txn;
use std::sync::Arc;

#[self_referencing(pub_extras)]
pub struct RawTable {
    pub mdbx_env: Arc<Database<WriteMap>>,

    #[borrows(mdbx_env)]
    #[covariant]
    pub mdbx_db: Table<'this>,
}

impl RawTable {
    pub fn get<'txn, TK: TransactionKind>(
        &'txn self,
        txn: &'txn Txn<'txn, TK>,
        k: &[u8],
    ) -> anyhow::Result<Option<Cow<'txn, [u8]>>> {
        txn.mdbx_txn
            .get::<Cow<'txn, [u8]>>(self.borrow_mdbx_db(), k.as_ref())
            .map_err(|e| e.into())
    }

    pub fn put<'txn>(
        &self,
        txn: &Txn<'txn, RW>,
        k: impl AsRef<[u8]>,
        v: impl AsRef<[u8]>,
    ) -> anyhow::Result<()> {
        txn.mdbx_txn.put(
            self.borrow_mdbx_db(),
            k.as_ref(),
            v.as_ref(),
            WriteFlags::empty(),
        )?;
        Ok(())
    }

    /// Returns true if the key was present.
    pub fn delete<'txn>(&self, txn: &Txn<'txn, RW>, k: impl AsRef<[u8]>) -> anyhow::Result<bool> {
        Ok(txn.mdbx_txn.del(self.borrow_mdbx_db(), k.as_ref(), None)?)
    }
}
