use crate::environment::Db;
use tempfile::NamedTempFile;

fn temp_env() -> Db {
    let tempfile = NamedTempFile::new().unwrap();
    Db::open(tempfile.path()).unwrap()
}

#[test]
fn test_raw_bytes_basics() {
    let env = temp_env();
    let db = env.open_raw_table(Some("rawt"), ()).unwrap();

    let k: [u8; 4] = [0, 1, 2, 3];

    {
        let txn = env.ro_txn().unwrap();
        assert_eq!(db.get(&txn, &k).unwrap(), None);
    }

    env.rw_txn(|txn| db.put(txn, &k, &[42, 41, 43])).unwrap();

    {
        let txn = env.ro_txn().unwrap();
        assert_eq!(
            db.get(&txn, &k).unwrap().unwrap().as_ref(),
            &[42, 41, 43][..]
        );
    }
}

#[test]
fn test_wrapped_str_basics() {
    let env = temp_env();
    let db = env
        .open_wrapped_table::<String, String>(Some("rawt"), ())
        .unwrap();

    {
        let txn = env.ro_txn().unwrap();
        assert_eq!(db.get(&txn, "ooh").unwrap(), None);
    }

    env.rw_txn(|txn| db.put(txn, "ooh", "cool!")).unwrap();

    {
        let txn = env.ro_txn().unwrap();
        assert_eq!(db.get(&txn, "ooh").unwrap().unwrap().as_str(), "cool!");
    }
}
